using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;
using System.Net;
using System.Net.Sockets;
using System.Threading;

namespace PAC_Transaction_System
{
    public partial class FormMain : Form
    {        
        SerialPort sp = new SerialPort();
        UInt16 CRC;
        UInt16[] Write_Frame;
        UInt16[] Read_Frame;
        byte[] Send_Frame;
        byte[] Receive_Frame;
        byte[] Acknowledge_Frame = new byte[2];
        string[] MyIP;
        bool hexdecSRAdd = true;
        bool hexdecSRData = true;
        bool hexdecSWAdd = true;
        bool hexdecSWData = true;
        bool hexdecTRAdd = true;
        bool hexdecTRData = true;
        bool hexdecTWAdd = true;
        bool hexdecTWData = true;

        public FormMain()
        {
            InitializeComponent();

            MyIP = Get_Master_ID(); // { MasterIP, MasterID };

            // Serial Read Tab
            dgvSRData.Rows.Add("");
            foreach (string pn in SerialPort.GetPortNames())
                cmbSRPort.Items.Add(pn);
            cmbSRPort.Text = cmbSRPort.Items[0].ToString();
            cmbSRRate.Text = cmbSRRate.Items[0].ToString();
            cmbSRType.Text = cmbSRType.Items[0].ToString();
            dgvSRRead.Rows.Add("", "", "", "", "", "", "", "", "", "");
            dgvSRReceive.Rows.Add("", "", "", "", "", "", "", "", "", "", "", "");
            // Serial Write Tab
            foreach (string pn in SerialPort.GetPortNames())
                cmbSWPort.Items.Add(pn);
            cmbSWPort.Text = cmbSWPort.Items[0].ToString();
            cmbSWRate.Text = cmbSWRate.Items[0].ToString();
            cmbSWType.Text = cmbSWType.Items[0].ToString();
            // TCP/IP Read Tab
            dgvTRData.Rows.Add("");
            cmbTRType.Text = cmbTRType.Items[0].ToString();
            // TCP/IP Write Tab
            cmbTWType.Text = cmbTWType.Items[0].ToString();

        }


        /**************************************************************************************************/
        /***** Serial Read Tab ****************************************************************************/
        /**************************************************************************************************/
        private void dgvSRData_RowsAdded(object sender, DataGridViewRowsAddedEventArgs e)
        {
            if(dgvSRData.Rows.Count > 1)
                for (int i = 0; i < dgvSRData.Rows.Count; i++)
                    dgvSRData.Rows[i].HeaderCell.Value = (i + 1).ToString();
        }

        private void btnSerialRead_Click(object sender, EventArgs e)
        {
            // UInt16[] Read_Frame = { DestinationID, SourceID, CMD, DestinationMemoryType, DesMemAdd_Lower,
            //                         DesMemAdd_Higher, SourceMemType, SouMemAdd_Lower, SouMemAdd_Higher,
            //                         DataLength, CRC_Higher, CRC_Lower};
            Read_Frame = new UInt16[10];
            Read_Frame[0] = (UInt16)nudSRID.Value;
            Read_Frame[1] = UInt16.Parse(MyIP[1]);
            Read_Frame[2] = 0;
            Read_Frame[3] = 0;
            byte[] Slave_Address = BitConverter.GetBytes((UInt16)nudSRAddress.Value);
            Read_Frame[4] = Slave_Address[0];
            Read_Frame[5] = Slave_Address[1];
            Read_Frame[6] = 0;
            Read_Frame[7] = 100;
            Read_Frame[8] = 0;
            Read_Frame[9] = (UInt16)nudSRLength.Value;
            
            CRC = CRC16_Ram(Read_Frame.Length, Read_Frame);
            byte[] CRC_Byte = BitConverter.GetBytes(CRC);

            Send_Frame = new byte[Read_Frame.Length + 2];
            byte[] Draft;
            for (int i = 0; i < Read_Frame.Length; i++)
            {
                Draft = BitConverter.GetBytes(Read_Frame[i]);
                Send_Frame[i] = Draft[0];
            }
            Send_Frame[Read_Frame.Length] = CRC_Byte[1];
            Send_Frame[Read_Frame.Length + 1] = CRC_Byte[0];

            Receive_Frame = new byte[(int)nudSRLength.Value + 9];

            Serial_Port_Initialize(cmbSRPort.Text, int.Parse(cmbSRRate.Text));
            sp.Open();
            sp.Write(Send_Frame, 0, Send_Frame.Length);
            Thread.Sleep(100);
            try
            {
                int Acknowledge = sp.Read(Receive_Frame, 0, Receive_Frame.Length);
                btnSRConnCheck.BackColor = Color.Green;
            }
            catch
            {
                btnSRConnCheck.BackColor = Color.Red;
                sp.Close();
                return;
            }
            sp.Close();

            dgvSRData.Rows.Clear();
            for (int i = 7; i < Receive_Frame.Length - 2; i++)
            {
                dgvSRData.Rows.Add(Receive_Frame[i].ToString());
            }
            if (!hexdecSRData)
            {
                dgvSRData.Columns[0].HeaderText = "0 - 255";
                btnSRDataHex.Text = "d";
                hexdecSRData = !hexdecSRData;
            }

            UInt16[] Receive_Frame_CRC = new UInt16[Receive_Frame.Length - 2];
            for (int i = 0; i < Receive_Frame.Length - 2; i++)
                Receive_Frame_CRC[i] = (UInt16)Receive_Frame[i];

            //if(CRC16_Ram(Receive_Frame_CRC.Length, Receive_Frame_CRC) == (Receive_Frame[Receive_Frame.Length-1]*256+Receive_Frame[Receive_Frame.Length-1]))
                btnSRCrcCheck.BackColor = Color.Green;
            //else
                //btnSRCrcCheck.BackColor = Color.Red;

            
            DataTable dtSRReceive = new DataTable();
            dtSRReceive.Columns.Add("DesID (PC)"); dtSRReceive.Columns.Add("SouID (PAC)"); dtSRReceive.Columns.Add("CMD)");
            dtSRReceive.Columns.Add("DesMemType"); dtSRReceive.Columns.Add("DesAddL"); dtSRReceive.Columns.Add("DesAddH");
            dtSRReceive.Columns.Add("DataLen");
            for (int i = 1; i < nudSRLength.Value+1; i++)
            {
                dtSRReceive.Columns.Add("Data"+i.ToString());
            }
            dtSRReceive.Columns.Add("CRC_H");
            dtSRReceive.Columns.Add("CRC_L");
            dgvSRReceive.Columns.Clear();
            dgvSRReceive.DataSource = dtSRReceive;
            string[] recdraft = new string[Receive_Frame.Length];
            for (int i = 0; i < Receive_Frame.Length; i++)
            {
                recdraft[i] = Receive_Frame[i].ToString();
            }
            dtSRReceive.Rows.Add(recdraft);
        }

        private void btnSerialRead_MouseDown(object sender, MouseEventArgs e)
        {
            btnSerialRead.Image = PAC_Transaction_System.Properties.Resources.arrow_down_green_224;
            btnSerialRead.BackgroundImage = PAC_Transaction_System.Properties.Resources.download_green;
            btnSRConnCheck.BackColor = Color.Black;
            btnSRCrcCheck.BackColor = Color.Black;        
        }

        private void btnSerialRead_MouseUp(object sender, MouseEventArgs e)
        {
            btnSerialRead.Image = PAC_Transaction_System.Properties.Resources.arrow_down_green_24;
            btnSerialRead.BackgroundImage = PAC_Transaction_System.Properties.Resources.download;       
        }
        
        private void btnSRRefresh_Click(object sender, EventArgs e)
        {
            cmbSRPort.Items.Clear();
            foreach (string pn in SerialPort.GetPortNames())
                cmbSRPort.Items.Add(pn);
            cmbSRPort.Text = cmbSRPort.Items[0].ToString();
        }

        private void btnSRAddHex_Click(object sender, EventArgs e)
        {
            if (hexdecSRAdd)
            {
                nudSRAddress.Hexadecimal = true;
                btnSRAddHex.Text = "x";
            }
            else
            {
                nudSRAddress.Hexadecimal = false;
                btnSRAddHex.Text = "d";
            }
            hexdecSRAdd = !(hexdecSRAdd);
        }

        private void btnSRDataHex_Click(object sender, EventArgs e)
        {
            if (dgvSRData.Rows.Count > 1)
            {
                dgvSRData.Rows.Add("");
                if (hexdecSRData)
                {
                    dec2hex(dgvSRData);
                    dgvSRData.Columns[0].HeaderText = "00 - FF";
                    btnSRDataHex.Text = "x";
                }
                else
                {
                    hex2dec(dgvSRData);
                    dgvSRData.Columns[0].HeaderText = "0 - 255";
                    btnSRDataHex.Text = "d";
                }
                hexdecSRData = !(hexdecSRData);
            }
        }

        private void btnSRView_Click(object sender, EventArgs e)
        {
            Read_Frame = new UInt16[10];
            Read_Frame[0] = (UInt16)nudSRID.Value;
            Read_Frame[1] = UInt16.Parse(MyIP[1]);
            Read_Frame[2] = 0;
            Read_Frame[3] = 0;
            byte[] Slave_Address = BitConverter.GetBytes((UInt16)nudSRAddress.Value);
            Read_Frame[4] = Slave_Address[0];
            Read_Frame[5] = Slave_Address[1];
            Read_Frame[6] = 0;
            Read_Frame[7] = 100;
            Read_Frame[8] = 0;
            Read_Frame[9] = (UInt16)nudSRLength.Value;

            CRC = CRC16_Ram(Read_Frame.Length, Read_Frame);
            byte[] CRC_Byte = BitConverter.GetBytes(CRC);

            Send_Frame = new byte[Read_Frame.Length + 2];
            byte[] Draft;
            for (int i = 0; i < Read_Frame.Length; i++)
            {
                Draft = BitConverter.GetBytes(Read_Frame[i]);
                Send_Frame[i] = Draft[0];
            }
            Send_Frame[Read_Frame.Length] = CRC_Byte[1];
            Send_Frame[Read_Frame.Length + 1] = CRC_Byte[0];

            dgvSRRead.Rows.Clear();
            dgvSRRead.Rows.Add(Send_Frame[0].ToString(), Send_Frame[1].ToString(), Send_Frame[2].ToString(),
                Send_Frame[3].ToString(), Send_Frame[4].ToString(), Send_Frame[5].ToString(), Send_Frame[6].ToString(),
                Send_Frame[7].ToString(), Send_Frame[8].ToString(), Send_Frame[9].ToString(), Send_Frame[10].ToString(),
                Send_Frame[11].ToString());
        }




        private void pnlSRHelp_Click(object sender, EventArgs e)
        {
            formHelp frm = new formHelp();
            frm.Show();
        }      
        /***** End of Serial Read Tab *********************************************************************/


        /**************************************************************************************************/
        /***** Serial Write Tab ***************************************************************************/
        /**************************************************************************************************/
        private void dgvSWData_RowsAdded(object sender, DataGridViewRowsAddedEventArgs e)
        {
            for (int i = 0; i < dgvSWData.Rows.Count - 1; i++)
                dgvSWData.Rows[i].HeaderCell.Value = (i + 1).ToString();
        }

        private void dgvSWData_RowsRemoved(object sender, DataGridViewRowsRemovedEventArgs e)
        {
            for (int i = 0; i < dgvSWData.Rows.Count - 1; i++)
                dgvSWData.Rows[i].HeaderCell.Value = (i + 1).ToString();        
        }

        private void btnSerialWrite_Click(object sender, EventArgs e)
        {
            // UInt16[] Write_Frame = { DestinationID, SourceID, CMD, DestinationMemoryType, DesMemAdd_Lower,
            //                         DesMemAdd_Higher, DataLength, Data, CRC_Higher, CRC_Lower};
            Write_Frame = new UInt16[dgvSWData.Rows.Count -1 + 7];
            Write_Frame[0] = (UInt16)nudSWID.Value;
            Write_Frame[1] = UInt16.Parse(MyIP[1]);
            Write_Frame[2] = 1;
            Write_Frame[3] = 0;
            byte[] Slave_Address = BitConverter.GetBytes((UInt16)nudSWAddress.Value);
            Write_Frame[4] = Slave_Address[0];
            Write_Frame[5] = Slave_Address[1];
            Write_Frame[6] = (UInt16)(dgvSWData.Rows.Count -1);
            
            if (hexdecSWData)
            {
                for (int i = 7; i < Write_Frame.Length; i++)
                    Write_Frame[i] = UInt16.Parse(dgvSWData.Rows[i - 7].Cells[0].Value.ToString());
            }
            else
            {
                for (int i = 7; i < Write_Frame.Length; i++)
                    Write_Frame[i] = Convert.ToUInt16(dgvSWData.Rows[i - 7].Cells[0].Value.ToString(), 16);
            }
            
            CRC = CRC16_Ram(Write_Frame.Length, Write_Frame);
            byte[] CRC_Byte = BitConverter.GetBytes(CRC);

            Send_Frame = new byte[Write_Frame.Length + 2];
            byte[] Draft;
            for (int i = 0; i < Write_Frame.Length; i++)
            {
                Draft = BitConverter.GetBytes(Write_Frame[i]);
                Send_Frame[i] = Draft[0];
            }
            Send_Frame[Write_Frame.Length] = CRC_Byte[1];
            Send_Frame[Write_Frame.Length + 1] = CRC_Byte[0];
            
            Serial_Port_Initialize(cmbSWPort.Text, int.Parse(cmbSWRate.Text));
            sp.Open();
            try
            {
                sp.Write(Send_Frame, 0, Send_Frame.Length);
                Thread.Sleep(300);
                int Acknowledge = sp.Read(Acknowledge_Frame, 0, 2);
                btnSWConnCheck.BackColor = Color.Green;
            }
            catch
            {
                btnSWConnCheck.BackColor = Color.Red;
                sp.Close();
                return;
            }
            sp.Close();

            if((int)Acknowledge_Frame[0] == int.Parse(MyIP[1]) & Acknowledge_Frame[1] == 2)
                btnSWAckCheck.BackColor = Color.Green;
            else
                btnSWAckCheck.BackColor = Color.Red;
        }

        private void btnSerialWrite_MouseDown(object sender, MouseEventArgs e)
        {
            btnSerialWrite.BackgroundImage = PAC_Transaction_System.Properties.Resources.download_blue;
            btnSWConnCheck.BackColor = Color.Black;
            btnSWAckCheck.BackColor = Color.Black;
        }

        private void btnSerialWrite_MouseUp(object sender, MouseEventArgs e)
        {
            btnSerialWrite.BackgroundImage = PAC_Transaction_System.Properties.Resources.download;
        }

        private void btnSWRefresh_Click(object sender, EventArgs e)
        {
            cmbSWPort.Items.Clear();
            foreach (string pn in SerialPort.GetPortNames())
                cmbSWPort.Items.Add(pn);
            cmbSWPort.Text = cmbSWPort.Items[0].ToString();
        }

        private void btnSWDataHex_Click(object sender, EventArgs e)
        {
            if (hexdecSWData)
            {
                dec2hex(dgvSWData);
                dgvSWData.Columns[0].HeaderText = "00 - FF";
                btnSWDataHex.Text = "x";
            }
            else
            {
                hex2dec(dgvSWData);
                dgvSWData.Columns[0].HeaderText = "0 - 255";
                btnSWDataHex.Text = "d";
            }
            hexdecSWData = !(hexdecSWData);
        }

        private void btnSWAddHex_Click(object sender, EventArgs e)
        {
            if (hexdecSWAdd)
            {
                nudSWAddress.Hexadecimal = true;
                btnSWAddHex.Text = "x";
            }
            else
            {
                nudSWAddress.Hexadecimal = false;
                btnSWAddHex.Text = "d";
            }
            hexdecSWAdd = !(hexdecSWAdd);
        }
        /***** End of Serial Write Tab *********************************************************************/


        /**************************************************************************************************/
        /***** TCP/IP Read Tab ****************************************************************************/
        /**************************************************************************************************/
        private void btnTcpRead_Click(object sender, EventArgs e)
        {
            // UInt16[] Read_Frame = { CMD, DestinationMemoryType, DesMemAdd_Lower, DesMemAdd_Higher,
            //                         SourceMemType, SouMemAdd_Lower, SouMemAdd_Higher, DataLength};

            Read_Frame = new UInt16[8];
            Read_Frame[0] = 0;
            Read_Frame[1] = 0;
            byte[] Slave_Address = BitConverter.GetBytes((UInt16)nudTRAddress.Value);
            Read_Frame[2] = Slave_Address[0];
            Read_Frame[3] = Slave_Address[1];
            Read_Frame[4] = 0;
            Read_Frame[5] = 0;
            Read_Frame[6] = 0;
            Read_Frame[7] = (UInt16)nudTRLength.Value;

            byte[] Send_Frame = new byte[Read_Frame.Length];
            byte[] Draft;
            for (int i = 0; i < Read_Frame.Length; i++)
            {
                Draft = BitConverter.GetBytes(Read_Frame[i]);
                Send_Frame[i] = Draft[0];
            }

            Receive_Frame = new byte[(int)nudTRLength.Value + 5];
            IPEndPoint tcpip = new IPEndPoint(IPAddress.Parse(MyIP[0]+nudTRID.Value.ToString()), 2000);
            Socket server = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);            
            try
            {
                server.Connect(tcpip);
                server.Send(Send_Frame, Send_Frame.Length, SocketFlags.None);
                int recv = server.Receive(Receive_Frame);
                btnTRConnCheck.BackColor = Color.Green;
            }
            catch (SocketException ee)
            {
                btnTRConnCheck.BackColor = Color.Red;
                //MessageBox.Show(ee.ToString(), "Error");;
                return;
            }
            server.Shutdown(SocketShutdown.Both);
            server.Close();
            
            dgvTRData.Rows.Clear();
            for (int i = 5; i < Receive_Frame.Length; i++)
            {
                dgvTRData.Rows.Add(Receive_Frame[i].ToString());
            }
            if (!hexdecTRData)
            {
                dgvTRData.Columns[0].HeaderText = "0 - 255";
                btnTRDataHex.Text = "d";
                hexdecTRData = !hexdecTRData;
            }
            

        }

        private void dgvTRData_RowsAdded(object sender, DataGridViewRowsAddedEventArgs e)
        {
            if(dgvTRData.Rows.Count > 1)
                for (int i = 0; i < dgvTRData.Rows.Count; i++)
                    dgvTRData.Rows[i].HeaderCell.Value = (i + 1).ToString();
        }

        private void btnTcpRead_MouseDown(object sender, MouseEventArgs e)
        {
            btnTcpRead.BackgroundImage = PAC_Transaction_System.Properties.Resources.Socket_green;
            btnTcpRead.Image = PAC_Transaction_System.Properties.Resources.arrow_down_green_224;
            btnTRConnCheck.BackColor = Color.Black;
        }

        private void btnTcpRead_MouseUp(object sender, MouseEventArgs e)
        {
            btnTcpRead.BackgroundImage = PAC_Transaction_System.Properties.Resources.Socket;
            btnTcpRead.Image = PAC_Transaction_System.Properties.Resources.arrow_down_green_24;
        }

        private void btnTRAddHex_Click(object sender, EventArgs e)
        {
            if (hexdecTRAdd)
            {
                nudTRAddress.Hexadecimal = true;
                btnTRAddHex.Text = "x";
            }
            else
            {
                nudTRAddress.Hexadecimal = false;
                btnTRAddHex.Text = "d";
            }
            hexdecTRAdd = !(hexdecTRAdd);
        }

        private void btnTRDataHex_Click(object sender, EventArgs e)
        {
            if (dgvTRData.Rows.Count > 1)
            {
                dgvTRData.Rows.Add("");
                if (hexdecTRData)
                {
                    dec2hex(dgvTRData);
                    dgvTRData.Columns[0].HeaderText = "00 - FF";
                    btnTRDataHex.Text = "x";
                }
                else
                {
                    hex2dec(dgvTRData);
                    dgvTRData.Columns[0].HeaderText = "0 - 255";
                    btnTRDataHex.Text = "d";
                }
                hexdecTRData = !(hexdecTRData);
            }
        }

        private void btnTRHelp_Click(object sender, EventArgs e)
        {

        }
        /***** End of TCP/IP Read Tab *********************************************************************/


        /**************************************************************************************************/
        /***** TCP/IP Write Tab ***************************************************************************/
        /**************************************************************************************************/
        private void btnTcpWrite_Click(object sender, EventArgs e)
        {
            // UInt16[] Write_Frame = { CMD, DestinationMemoryType, DesMemAdd_Lower, DesMemAdd_Higher, DataLength, Data};
            Write_Frame = new UInt16[5 + dgvTWData.Rows.Count - 1];
            Write_Frame[0] = 1;
            Write_Frame[1] = 0;
            byte[] Slave_Address = BitConverter.GetBytes((UInt16)nudTWAddress.Value);
            Write_Frame[2] = Slave_Address[0];
            Write_Frame[3] = Slave_Address[1];
            Write_Frame[4] = (UInt16)(dgvTWData.Rows.Count - 1);

            if (hexdecTWData)
            {
                for (int i = 5; i < Write_Frame.Length; i++)
                    Write_Frame[i] = UInt16.Parse(dgvTWData.Rows[i - 5].Cells[0].Value.ToString());
            }
            else
            {
                for (int i = 5; i < Write_Frame.Length; i++)
                    Write_Frame[i] = Convert.ToUInt16(dgvTWData.Rows[i - 5].Cells[0].Value.ToString(), 16);
            }
            

            Send_Frame = new byte[Write_Frame.Length];
            byte[] Draft;
            for (int i = 0; i < Write_Frame.Length; i++)
            {
                Draft = BitConverter.GetBytes(Write_Frame[i]);
                Send_Frame[i] = Draft[0];
            }
            IPEndPoint tcpip = new IPEndPoint(IPAddress.Parse(MyIP[0]+nudTWID.Value.ToString()), 2000);
            Socket server = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);                       
            try
            {
                server.Connect(tcpip);
            }
            catch (SocketException ee)
            {
                btnTWConnCheck.BackColor = Color.Red;
                //MessageBox.Show(ee.ToString(), "Error");
                return;
            }
            btnTWConnCheck.BackColor = Color.Green;

            server.Send(Send_Frame, Send_Frame.Length, SocketFlags.None);
            int recv = server.Receive(Acknowledge_Frame);
            server.Shutdown(SocketShutdown.Both);
            server.Close();

            if (Acknowledge_Frame[0] == 2)
                btnTWAckCheck.BackColor = Color.Green;
            else
                btnTWAckCheck.BackColor = Color.Red;


        }

        private void btnTcpWrite_MouseDown(object sender, MouseEventArgs e)
        {
            btnTcpWrite.BackgroundImage = PAC_Transaction_System.Properties.Resources.Socket_blue;
            btnTWConnCheck.BackColor = Color.Black;
            btnTWAckCheck.BackColor = Color.Black;
        }
        
        private void btnTcpWrite_MouseUp(object sender, MouseEventArgs e)
        {

            btnTcpWrite.BackgroundImage = PAC_Transaction_System.Properties.Resources.Socket_Tcp;
        }  

        private void btnTWAddHex_Click(object sender, EventArgs e)
        {
            if (hexdecTWAdd)
            {
                nudTWAddress.Hexadecimal = true;
                btnTWAddHex.Text = "x";
            }
            else
            {
                nudTWAddress.Hexadecimal = false;
                btnTWAddHex.Text = "d";
            }
            hexdecTWAdd = !(hexdecTWAdd);
        }

        private void btnTWDataHex_Click(object sender, EventArgs e)
        {
            if (hexdecTWData)
            {
                dec2hex(dgvTWData);
                dgvTWData.Columns[0].HeaderText = "00 - FF";
                btnTWDataHex.Text = "x";
            }
            else
            {
                hex2dec(dgvTWData);
                dgvTWData.Columns[0].HeaderText = "0 - 255";
                btnTWDataHex.Text = "d";
            }
            hexdecTWData = !(hexdecTWData);
        }

        private void btnTWHelp_Click(object sender, EventArgs e)
        {

        }

        private void dgvTWData_RowsAdded(object sender, DataGridViewRowsAddedEventArgs e)
        {
            for (int i = 0; i < dgvTWData.Rows.Count-1; i++)
                dgvTWData.Rows[i].HeaderCell.Value = (i + 1).ToString();
        }

        private void dgvTWData_RowsRemoved(object sender, DataGridViewRowsRemovedEventArgs e)
        {
            for (int i = 0; i < dgvTWData.Rows.Count - 1; i++)
                dgvTWData.Rows[i].HeaderCell.Value = (i + 1).ToString();
        }        
        /***** End of TCP/IP Write Tab ********************************************************************/


        /**************************************************************************************************/
        /***** General Functions (Get_Master_ID, CRC16_Ram, Serial_Port_Initialize, hex2dec, dec2hex) *****/
        /**************************************************************************************************/
        private string[] Get_Master_ID()
        {
            string myip = Dns.GetHostByName(Dns.GetHostName()).AddressList[0].ToString();
            int dotcount = 0;
            string MasID = "";
            string MasIP = "";
            for (int i = 1; i < myip.Length; i++)
            {
                if (myip[i] == '.')
                    dotcount++;
                if (dotcount == 3)
                {
                    MasID = myip.Substring(i + 1, (myip.Length - i - 1));
                    MasIP = myip.Remove(i + 1);
                    break;
                }
            }
            string[] returnbyte = { MasIP, MasID };
            return (returnbyte);
        }

        private UInt16 CRC16_Ram(int Length, UInt16[] Pointer)
        {
            CRC = 0xFFFF;
            UInt16 i;
            UInt16[] CrcTable = { 
                0X0000, 0XC1C0, 0X81C1, 0X4001, 0X01C3, 0XC003, 0X8002, 0X41C2,  
                0X01C6, 0XC006, 0X8007, 0X41C7, 0X0005, 0XC1C5, 0X81C4, 0X4004,  
                0X01CC, 0XC00C, 0X800D, 0X41CD, 0X000F, 0XC1CF, 0X81CE, 0X400E,  
                0X000A, 0XC1CA, 0X81CB, 0X400B, 0X01C9, 0XC009, 0X8008, 0X41C8,  
                0X01D8, 0XC018, 0X8019, 0X41D9, 0X001B, 0XC1DB, 0X81DA, 0X401A,  
                0X001E, 0XC1DE, 0X81DF, 0X401F, 0X01DD, 0XC01D, 0X801C, 0X41DC,  
                0X0014, 0XC1D4, 0X81D5, 0X4015, 0X01D7, 0XC017, 0X8016, 0X41D6,  
                0X01D2, 0XC012, 0X8013, 0X41D3, 0X0011, 0XC1D1, 0X81D0, 0X4010,  
                0X01F0, 0XC030, 0X8031, 0X41F1, 0X0033, 0XC1F3, 0X81F2, 0X4032,  
                0X0036, 0XC1F6, 0X81F7, 0X4037, 0X01F5, 0XC035, 0X8034, 0X41F4,  
                0X003C, 0XC1FC, 0X81FD, 0X403D, 0X01FF, 0XC03F, 0X803E, 0X41FE,  
                0X01FA, 0XC03A, 0X803B, 0X41FB, 0X0039, 0XC1F9, 0X81F8, 0X4038,  
                0X0028, 0XC1E8, 0X81E9, 0X4029, 0X01EB, 0XC02B, 0X802A, 0X41EA,  
                0X01EE, 0XC02E, 0X802F, 0X41EF, 0X002D, 0XC1ED, 0X81EC, 0X402C,  
                0X01E4, 0XC024, 0X8025, 0X41E5, 0X0027, 0XC1E7, 0X81E6, 0X4026,  
                0X0022, 0XC1E2, 0X81E3, 0X4023, 0X01E1, 0XC021, 0X8020, 0X41E0,  
                0X01A0, 0XC060, 0X8061, 0X41A1, 0X0063, 0XC1A3, 0X81A2, 0X4062,  
                0X0066, 0XC1A6, 0X81A7, 0X4067, 0X01A5, 0XC065, 0X8064, 0X41A4,  
                0X006C, 0XC1AC, 0X81AD, 0X406D, 0X01AF, 0XC06F, 0X806E, 0X41AE,  
                0X01AA, 0XC06A, 0X806B, 0X41AB, 0X0069, 0XC1A9, 0X81A8, 0X4068,  
                0X0078, 0XC1B8, 0X81B9, 0X4079, 0X01BB, 0XC07B, 0X807A, 0X41BA,  
                0X01BE, 0XC07E, 0X807F, 0X41BF, 0X007D, 0XC1BD, 0X81BC, 0X407C,  
                0X01B4, 0XC074, 0X8075, 0X41B5, 0X0077, 0XC1B7, 0X81B6, 0X4076,  
                0X0072, 0XC1B2, 0X81B3, 0X4073, 0X01B1, 0XC071, 0X8070, 0X41B0,  
                0X0050, 0XC190, 0X8191, 0X4051, 0X0193, 0XC053, 0X8052, 0X4192,  
                0X0196, 0XC056, 0X8057, 0X4197, 0X0055, 0XC195, 0X8194, 0X4054,  
                0X019C, 0XC05C, 0X805D, 0X419D, 0X005F, 0XC19F, 0X819E, 0X405E,  
                0X005A, 0XC19A, 0X819B, 0X405B, 0X0199, 0XC059, 0X8058, 0X4198,  
                0X0188, 0XC048, 0X8049, 0X4189, 0X004B, 0XC18B, 0X818A, 0X404A,  
                0X004E, 0XC18E, 0X818F, 0X404F, 0X018D, 0XC04D, 0X804C, 0X418C,  
                0X0044, 0XC184, 0X8185, 0X4045, 0X0187, 0XC047, 0X8046, 0X4186,  
                0X0182, 0XC042, 0X8043, 0X4183, 0X0041, 0XC181, 0X8180, 0X4040 };

            for (i = 0; i < Length; i++)
            {
                CRC = (UInt16)((CRC << 8) ^ CrcTable[(CRC >> 8) ^ Pointer[i]]);
            }
            return (CRC);
        }

        private void Serial_Port_Initialize(string PortName, int BaudRate)
        {
            sp.PortName = PortName;
            sp.BaudRate = BaudRate;
            sp.Parity = System.IO.Ports.Parity.None;
            sp.StopBits = System.IO.Ports.StopBits.One;
            sp.DataBits = 8;
            sp.Handshake = System.IO.Ports.Handshake.None;
            sp.ReadTimeout = 300;
            sp.WriteTimeout = 300;
        }

        private void hex2dec(DataGridView dgv)
        {
            string[] newstr = new string[dgv.Rows.Count - 1];
            int[] newint = new int[dgv.Rows.Count - 1];
            int dgvcount = dgv.Rows.Count;

            for (int i = 0; i < dgvcount - 1; i++)
            {
                newstr[i] = dgv.Rows[i].Cells[0].Value.ToString();
                newint[i] = Convert.ToInt32(newstr[i], 16);
            }
            dgv.Rows.Clear();
            for (int i = 0; i < dgvcount - 1; i++)
            {
                dgv.Rows.Add(newint[i].ToString());
            }
            if (dgvcount == 0)
                dgv.Rows.Add("");
        }

        private void dec2hex(DataGridView dgv)
        {
            string[] newstr = new string[dgv.Rows.Count - 1];
            int[] newint = new int[dgv.Rows.Count - 1];
            int dgvcount = dgv.Rows.Count;

            for (int i = 0; i < dgvcount - 1; i++)
            {
                newstr[i] = dgv.Rows[i].Cells[0].Value.ToString();
                newint[i] = int.Parse(newstr[i]);
            }
            dgv.Rows.Clear();
            for (int i = 0; i < dgvcount - 1; i++)
            {
                dgv.Rows.Add(newint[i].ToString("X"));
            }
        }

        private void pnlSWHelp_Click(object sender, EventArgs e)
        {
            formHelp frm = new formHelp();
            frm.Show();
        }

        private void pnlTRHelp_Click(object sender, EventArgs e)
        {
            formHelp frm = new formHelp();
            frm.Show();
        }

        
    }
}